__title__ = "sadedegel"  # pragma: no cover
__version__ = "0.18.3"  # pragma: no cover
__release__ = True  # pragma: no cover
__download_url__ = "https://github.com/globalmaksimum/sadedegel/releases"  # pragma: no cover
__herokuapp_url__ = "https://sadedegel.herokuapp.com"  # pragma: no cover
